/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file toolbar.h
 *
 * @type subsystem
 *
 * @description
 *
 * The toolbar subsystem provides a toolbar system, where a canvas can have a
 * number of toolbars attached to it in different locations, moving with the
 * main canvas. The toolbar subsystem also provides toolbar contexts, where
 * a toolbar can change context, such as context sensitive tools. Context
 * sensitive tool windows are achieved by scrolling them for each of the
 * contexts.
 *
 * There are also a number of built-in icons for use with the toolbar system,
 * arranged in such a way as to only build in the icons that are in use.
 *
 * Jut to add to the confusion, a window that allows toolbars is classed a
 * toolbar. A window inside a toolbar is a toolwin.
 */

#ifndef __rov_toolbar
#define __rov_toolbar


#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @section Built in icons
 */
extern char toolspr_bass[],
	 toolspr_bin[],
	 toolspr_bold[],
	 toolspr_copy[],
	 toolspr_crochet[],
	 toolspr_cross[],
	 toolspr_cut[],
	 toolspr_down[],
	 toolspr_end[],
	 toolspr_erase[],
	 toolspr_ffwd[],
	 toolspr_flat[],
	 toolspr_insert[],
	 toolspr_italic[],
	 toolspr_left[],
	 toolspr_lips[],
	 toolspr_mail[],
	 toolspr_minus[],
	 toolspr_paste[],
	 toolspr_phone[],
	 toolspr_play[],
	 toolspr_plus[],
	 toolspr_pointer[],
	 toolspr_print[],
	 toolspr_quaver[],
	 toolspr_record[],
	 toolspr_redo[],
	 toolspr_resize[],
	 toolspr_right[],
	 toolspr_rotate[],
	 toolspr_rwnd[],
	 toolspr_save[],
	 toolspr_send[],
	 toolspr_sharp[],
	 toolspr_start[],
	 toolspr_stop[],
	 toolspr_sum[],
	 toolspr_text[],
	 toolspr_tick[],
	 toolspr_treble[],
	 toolspr_underline[],
	 toolspr_undo[],
	 toolspr_up[],
	 toolspr_write[],
	 toolspr_zoom[];

/**
 * Definition of the toolbar redraw event handler
 * @param win the tool window
 * @param x0 the (canvas) left hand coordinate.
 * @param y0 the (canvas) bottom coordinate.
 * @param x1 the (canvas) right hand coordinate.
 * @param y1 the (canvas) top coordinate.
 */
typedef void (*toolbar_redrawp)(void *win,
				int x0,
				int y0,
				int x1,
				int y1);

/**
 * The definition of a toolbar.
 */
typedef void * toolbar;

/**
 * The definition of a tool window.
 */
typedef void * toolwin;

/* Create a toolbar.
 * @param x0 the minimum x coordinate.
 * @param y0 the minimum y coordinate.
 * @param x1 the maximum x coordinate.
 * @param y1 the maximum y coordinate.
 * @param ctxs the number of contexts.
 * @param title the title  for the toolbar.
 * @return the created tool
 * @note Like a canvas, the toolbar is not turned into a window
 * until it is made.
 * @see toolbar_make()
 * @see toolbar_makeatlocation()
 * @see toolbar_makeatlocationsize()
 */
toolbar toolbar_create(int x0,
		       int y0,
		       int x1,
		       int y1,
		       int ctxs,
		       char* title);

/**
 * Make a toolbar into a window.
 * @param tb the toolbar to make.
 */
void toolbar_make(toolbar tb);

/**
 * Make a toolbar at a particular location.
 * @param tb the toolbar to make.
 * @param x the (screen) left hand coordinate.
 * @param y the (screen) right hand coordinate.
 * @see toolbar_makeatlocation()
 * @see toolbar_makeatlocationsize()
 */
void toolbar_makeatlocation(toolbar tb,
			    int x,
			    int y);

/**
 * Make a toolbar in a given bounding box.
 * @param tb the toolbar to make.
 * @param x0 the (screen) left hand coordinate.
 * @param y0 the (screen) bottom coordinate.
 * @param x1 the (screen) right hand coordinate.
 * @param y1 the (screen) top coordinate.
 * @see toolbar_make()
 * @see toolbar_makeatlocation()
 */
void toolbar_makeatlocationsize(toolbar tb,
				int x0,
				int y0,
				int x1,
				int y1);

/**
 * Create a tool window on a toolbar.
 * @param tb the toolbar to create the tool window on.
 * @param flags the tool window flags.
 * @param size the size of the toolbar (zero makes a standard width toolbar).
 * @return the created tool window.
 * @note The tool window is not turned into a window until it has been made.
 * @see toolbar_winmake()
 * @see Tool window flags
 * @see Tool window shortcuts
 */
toolwin toolbar_window(toolbar tb,
		       int flags,
		       int size);

/**
 * Make a tool window a window
 * @param tw the tool window.
 */
void toolbar_winmake(toolwin tw);

/**
 * Get the canvas for a tool window.
 * @param tw the tool window.
 * @return the canvas.
 * @note Do not claim any handlers for this canvas, and do not use the
 * canvas ID handle.
 */
canvas toolbar_winhandle(toolwin tw);

/**
 * Get the canvas for a toolbar.
 * @param tb the toolbar.
 * @return the canvas.
 * @note Do not claim any handlers for this canvas, and do not use the
 * canvas ID handle.
 */
canvas toolbar_handle(toolbar tb);

/**
 * Claim redraw handlers for a tool window.
 * @param tw the tool window.
 * @param pre_handler the handler called before the toolbar system
 * performs it redrawing.
 * @param post_handler the handler called after the toolbar system
 * performs its redrawing.
 * @see toolbar_redrawp
 */
void toolbar_winredraw(toolwin tw,
		       toolbar_redrawp pre_handler,
		       toolbar_redrawp post_handler);

/**
 * Add a sprite to the toolbar, and move the current position on.
 * @param tw the tool window.
 * @param flags the tool icon flags.
 * @param ctx the context to which this icon belongs.
 * @param spr_area the pointer to the sprite area.
 * @param spr_name the name of the sprite.
 * @return the generated icon.
 * @note if the sprite name is called "abc", then the sprite "abc0" is
 * the 1x1 aspect sprite, "pabc0" is the 1x1 aspect pressed sprite,
 * "abc1" is the 1x2 aspect sprite, and "pabc1" is the 1x2 aspect
 * preseed sprite.
 */
icon toolbar_addicon(toolwin tw,
		     int flags,
		     int ctx,
		     void *spr_area,
		     char *spr_name);

/**
 * Leave a gap for the next icon in the tool window context.
 * @param tw the tool window.
 * @param ctx the context.
 * @param w the width of the gap.
 * @see toolbar_leavedefaultgap()
 */
void toolbar_leavegap(toolwin tw,
		      int ctx,
		      int w);

/**
 * Leave a default gap of 12 OS units.
 * @mparam aaa the tool window.
 * @mparam bbb the context.
 * @see toolbar_leavegap()
 */
#define toolbar_leavedefaultgap(aaa,bbb) toolbar_leavegap(aaa, bbb, 12)

/**
 * Set the size of a toolbar.
 * @param tb the toolbar.
 * @param x0 the minimum x coordinate.
 * @param y0 the minimum y coordinate.
 * @param x1 the maximum x coordinate.
 * @param y1 the maximum y coordinate.
 */
void toolbar_setsize(toolbar tb,
		     int x0,
		     int y0,
		     int x1,
		     int y1);

/**
 * Reopen a toolbar at the top of the window stack in its current
 * coordinates.
 * @param tb the toolbar.
 * @see toolbar_fullopen()
 * @see toolbar_opencentre()
 * @see toolbar_open()
 * @see toolbar_wimpopen()
 * @see toolbar_scrollopen()
 */
void toolbar_reopen(toolbar tb);

/**
 * Open a toolbar to its fullest extent.
 * @param tb the toolbar.
 * @see toolbar_reopen()
 * @see toolbar_opencentre()
 * @see toolbar_open()
 * @see toolbar_wimpopen()
 * @see toolbar_scrollopen()
 */
void toolbar_fullopen(toolbar tb);

/**
 * Open a toolbar at the top of the window stack in the centre of
 * the screen.
 * @see toolbar_reopen()
 * @see toolbar_fullopen()
 * @see toolbar_open()
 * @see toolbar_wimpopen()
 * @param tb the toolbar.
 * @see toolbar_scrollopen()
 */
void toolbar_opencentre(toolbar tb);

/**
 * Open a toolbar at the top of the window stack using given
 * coordinates.
 * @param tb the toolbar.
 * @param x the (screen) left hand coordinate.
 * @param y the (screen) bottom coordinate.
 * @see toolbar_reopen()
 * @see toolbar_fullopen()
 * @see toolbar_opencentre()
 * @see toolbar_wimpopen()
 * @see toolbar_scrollopen()
 */
void toolbar_open(toolbar tb,
		  int x,
		  int y);

/**
 * Open a toolbar using given canvas open parameters.
 * @param tb the toolbar.
 * @param open_str the canvas open window buffer.
 * @see toolbar_reopen()
 * @see toolbar_fullopen()
 * @see toolbar_open()
 * @see toolbar_opencentre()
 * @see toolbar_scrollopen()
 */
void toolbar_wimpopen(toolbar tb,
		      canvas_openstr *open_str);

/**
 * Cause a toolbar to change scroll location.
 * @param tb the toolbar.
 * @param open_str the canvas open window buffer.
 * @note Unlike toolbar_wimopen(), this does not move any windows around - it is only
 * used to change the scroll locations, and as a result is much faster.
 * @see toolbar_reopen()
 * @see toolbar_fullopen()
 * @see toolbar_open()
 * @see toolbar_opencentre()
 * @see toolbar_wimpopen()
 */
void toolbar_scrollopen(toolbar tb,
			canvas_openstr *open_str);

/**
 * Scroll a toolbar around.
 * @param tb the toolbar.
 * @param lr the left/right movement.
 * @param ud the up/down movement.
 */
void toolbar_pagescroll(toolbar tb,
			int lr,
			int ud);

/**
 * Get the coordinate of the context.
 * @param tw the window.
 * @param ctx the context.
 * @return the x or y coordinate of the base of a context.
 * @note The returned value depends  on whether it is a horizontal or
 * vertical tool window.
 * @note This location is used to allow you to create your own icons
 * instead of the standard toolbar icon system.
 */
int toolbar_coordinate(toolwin tw,
		       int ctx);

/**
 * Set the current toolbar context
 * @param tb the toolbar.
 * @param ctx the context to use
 */
void toolbar_setcontext(toolbar tb,
			int ctx);

/**
 * Close a toolbar.
 * @param tb the toolbar to close.
 */
void toolbar_close(toolbar tb);

/**
 * Make a tool window invisible.
 * @param tw the tool window.
 * @param flag non-zero is visible, zero is invisible.
 * @note This is used to make tool windows that appear when necessary, such
 * as an attachement tool window in an email system that only appears if
 * attachements are present in a message.
 */
void toolbar_winvisible(toolwin tw,
			int flag);

/**
 * Add a handler to a toolbar.
 * @param tb the toolbar.
 * @param code the toolbar handler event type
 * @param ... the handler function pointer.
 * @see toolbar_closehandler()
 * @see toolbar_resizehandler()
 * @see Toolbar event types
 */
void toolbar_addhandler(toolbar tb,
			int code,
			...);

/**
 * Remove the horizontal scroll bar from a toolbar.
 * @param tb the toolbar.
 * @note This must be done before the toolbar is made.
 * @see toolbar_removevscroll()
 */
void toolbar_removehscroll(toolbar tb);

/**
 * Remove the vertical scroll bar from a toolbar.
 * @param tb the toolbar.
 * @note This must be done before the toolbar is made.
 * @see toolbar_removehscroll()
 */
void toolbar_removevscroll(toolbar tb);

/**
 * @section Tool window flags
 */
#define TOOL_TOP	(0x00) /* Tool window at top */
#define TOOL_RIGHT	(0x01) /* Tool window at right */
#define TOOL_BOTTOM	(0x02) /* Tool window at bottom */
#define TOOL_LEFT	(0x03) /* Tool window at left */
#define TOOL_STATIC	(0x00) /* Tool window does not scroll with main window */
#define TOOL_SCROLL	(0x04) /* Tool window scrolls with main window */
#define TOOL_NOCONTEXT	(0x00) /* Tool window is context insensitive */
#define TOOL_CONTEXT	(0x08) /* Tool window is context sensitive */
#define TOOL_NOVSCROLL	(0x00) /* Tool window has no vertical scroll bar */
#define TOOL_VSCROLL	(0x10) /* Tool window has vertical scroll bar */
#define TOOL_NOHSCROLL	(0x00) /* Tool window has no horizontal scroll bar */
#define TOOL_HSCROLL	(0x20) /* Tool window has horizontal scroll bar */

/**
 * @section Tool window shortcuts
 */
#define TOOL_SCROLLTOP    (TOOL_TOP|TOOL_SCROLL)
#define TOOL_SCROLLRIGHT  (TOOL_RIGHT|TOOL_SCROLL)
#define TOOL_SCROLLBOTTOM (TOOL_BOTTOM|TOOL_SCROLL)
#define TOOL_SCROLLLEFT   (TOOL_LEFT|TOOL_SCROLL)
#define TOOL_STATICTOP    (TOOL_TOP|TOOL_STATIC)
#define TOOL_STATICRIGHT  (TOOL_RIGHT|TOOL_STATIC)
#define TOOL_STATICBOTTOM (TOOL_BOTTOM|TOOL_STATIC)
#define TOOL_STATICLEFT   (TOOL_LEFT|TOOL_STATIC)

/**
 * @section Tool window icon types
 */
#define TOOL_CLICK        (0x00)
#define TOOL_TOGGLE	  (0x01)

/**
 * @section Toolbar event types
 */
#define TOOLBAR_RESIZEHANDLER		(0)
#define TOOLBAR_CLOSEHANDLER		(1)

/**
 * Definition of a toolbar resize handler.
 * @param tb the toolbar.
 * @param w the new width.
 * @param h the new height.
 */
typedef void (*toolbar_resizehandler)(toolbar tb
				      int w,
				      int h);

/**
 * Definition of a toolbar close handler.
 * @param tb the toolbar.
 */
typedef void (*toolbar_closehandler)(toolbar tb);

/**
 * @section Tool sprites shortcuts
 * @note to use this, call toolbar_addicon() with the required icon in place
 * of the last two parameters.
 * @see toolbar_addicon()
 */
#define TOOL_BASS	toolspr_bass, "bass"
#define TOOL_BIN	toolspr_bin, "bin"
#define TOOL_COPY	toolspr_copy, "copy"
#define TOOL_CROSS	toolspr_cross, "cross"
#define TOOL_CUT	toolspr_cut, "cut"
#define TOOL_FLAT	toolspr_flat, "flat"
#define TOOL_PASTE	toolspr_paste, "paste"
#define TOOL_POINTER	toolspr_pointer, "pointer"
#define TOOL_PRINT	toolspr_print, "print"
#define TOOL_QUAVER	toolspr_quaver, "quaver"
#define TOOL_RECORD	toolspr_record, "record"
#define TOOL_REDO	toolspr_redo, "redo"
#define TOOL_RESIZE	toolspr_resize, "resize"
#define TOOL_ROTATE	toolspr_rotate, "rotate"
#define TOOL_SAVE	toolspr_save, "save"
#define TOOL_SEND	toolspr_send, "send"
#define TOOL_SHARP	toolspr_sharp, "sharp"
#define TOOL_SUM	toolspr_sum, "sum"
#define TOOL_TEXT	toolspr_text, "text"
#define TOOL_TICK	toolspr_tick, "tick"
#define TOOL_TREBLE	toolspr_treble, "treble"
#define TOOL_UNDO	toolspr_undo, "undo"
#define TOOL_WRITE	toolspr_write, "write"
#define TOOL_ZOOM	toolspr_zoom, "zoom"
#define TOOL_BOLD	toolspr_bold, "bold"
#define TOOL_DOWNI	toolspr_down, "down"
#define TOOL_END	toolspr_end, "end"
#define TOOL_ERASE	toolspr_erase, "erase"
#define TOOL_FFWD	toolspr_ffwd, "ffwd"
#define TOOL_INSERT	toolspr_insert, "insert"
#define TOOL_ITALIC	toolspr_italic, "italic"
#define TOOL_LEFTI	toolspr_left, "left"
#define TOOL_PLAY	toolspr_play, "play"
#define TOOL_RIGHTI	toolspr_right, "right"
#define TOOL_RWND	toolspr_rwnd, "rwnd"
#define TOOL_START	toolspr_start, "start"
#define TOOL_STOP	toolspr_stop, "stop"
#define TOOL_UPI	toolspr_up, "up"
#define TOOL_UNDERLINE	toolspr_underline, "underline"
#define TOOL_PLUS	toolspr_plus, "plus"
#define TOOL_MINUS	toolspr_minus, "minus"
#define TOOL_MAIL	toolspr_mail, "mail"
#define TOOL_PHONE	toolspr_phone, "phone"
#define TOOL_LIPS	toolspr_lips, "lips"


#ifdef __cplusplus
}
#endif

#endif
